/* $Id: twfwriter.h,v 1.26 2000/02/18 17:59:03 asv Exp $ */
/****************************************************************************
*                                                                           *
*  TWFWRITER.H                                                              *
*                                                                           *
*  Header of the library used to write TWF files.                           *
*                                                                           *
*  August 1999.                                                             *
*                                                                           *
*****************************************************************************/
/*
    TWF Writer - A Library to Easily and Programmatically Create TWF Files
    Copyright (C) 1999  TecGraf (http://www.tecgraf.puc-rio.br)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef TWFWRITER_H
#define TWFWRITER_H

#ifdef __cplusplus
extern "C" {
#endif

/* Return codes. */
#define TWF_OK		0	/* no errors found */
#define TWF_ERROR	-1	/* error writing the primitve */
#define TWF_DISCARD	-2	/* primitive was discarded by quantization */

/* Boolean values */
#define TWF_TRUE	1
#define TWF_FALSE	0

/* TWF file format limits */
#define TWF_SIZE_MAX		4194303L
#define TWF_IDENTIFIER_MAX	4194303L
#define TWF_SIDENTIFIER_MIN	-2097152L
#define TWF_SIDENTIFIER_MAX	2097151L

#define TWF_SHAPE_MAX		TWF_IDENTIFIER_MAX
#define TWF_IMAGE_MAX		TWF_IDENTIFIER_MAX
#define TWF_FACE_MAX		TWF_IDENTIFIER_MAX
#define TWF_EDGE_MAX		(TWF_SIDENTIFIER_MAX-1)

/* Error codes */
enum TWF_ERROR_CODE {
	TWF_ERR_NONE,		/* no error */
	TWF_ERR_SYSTEM,		/* system error (check errno) */
	TWF_ERR_LIMITS,		/* TWF file format limits exceeded */
	TWF_ERR_CONTEXT,	/* invalid context for operation */
	TWF_ERR_PARAM,		/* invalid parameter value */
	TWF_ERR_INTERNAL	/* internal library error */
};

/* Primitive types for TWF_begin */
enum TWF_BEGIN_TYPE {
	TWF_BEGIN_POLYLINE,
	TWF_BEGIN_POLYGON,
	TWF_BEGIN_CUBIC,
};

/* Color mode parameters for TWF_setColorSize */
enum TWF_COLOR_MODE {
 TWF_COLOR_INDEX1,
 TWF_COLOR_INDEX8,
 TWF_COLOR_GRAY8,
 TWF_COLOR_GRAY12,
 TWF_COLOR_GRAY16,
 TWF_COLOR_GA,
 TWF_COLOR_RGB,
 TWF_COLOR_RGBA
};

/* Mark type */
enum TWF_MARK_TYPE {
 TWF_MARK_PLUS,
 TWF_MARK_STAR,
 TWF_MARK_CIRCLE,
 TWF_MARK_X,
 TWF_MARK_BOX,
 TWF_MARK_DIAMOND,
 TWF_MARK_HOLLOW_CIRCLE,
 TWF_MARK_HOLLOW_BOX,
 TWF_MARK_HOLLOW_DIAMOND
};

/* Image stretch mode */
enum TWF_STRETCH_TYPE {
 TWF_STRETCH_NEAREST,
 TWF_STRETCH_BILINEAR
};

/* Font type */
enum TWF_FONT_TYPEFACE {
 TWF_FONT_DEFAULT,
 TWF_FONT_MONOSPACED,
 TWF_FONT_SERIF,
 TWF_FONT_SANS_SERIF
};

/* Font style */
enum TWF_FONT_STYLE {
 TWF_FONT_PLAIN,
 TWF_FONT_BOLD,
 TWF_FONT_ITALIC,
 TWF_FONT_BOLD_ITALIC
};

/* Text alignment */
enum TWF_TEXT_ALIGNMENT {
 TWF_TEXT_NORTH,
 TWF_TEXT_SOUTH,
 TWF_TEXT_EAST,
 TWF_TEXT_WEST,
 TWF_TEXT_NORTHEAST,
 TWF_TEXT_NORTHWEST,
 TWF_TEXT_SOUTHEAST,
 TWF_TEXT_SOUTHWEST,
 TWF_TEXT_CENTER,
 TWF_TEXT_BASELEFT,
 TWF_TEXT_BASECENTER,
 TWF_TEXT_BASERIGHT
};

/* Line joint style */
enum TWF_JOINT_STYLE {
 TWF_JOINT_DEFAULT,
 TWF_JOINT_MITER,
 TWF_JOINT_ROUND,
 TWF_JOINT_BEVEL
};

/* Start and end line style */
enum TWF_LINE_DECORATION_STYLE {
 TWF_NO_ARROW,
 TWF_ARROW,
 TWF_OPEN_ARROW,
 TWF_STEALTH_ARROW,
 TWF_DIAMOND_ARROW,
 TWF_OVAL_ARROW
};

/* Blend mode values for TWF_setBlendMode */
enum TWF_BLEND_MODE {
 TWF_BLEND_CLEAR,
 TWF_BLEND_SOURCE,
 TWF_BLEND_SOURCE_OVER_DEST,
 TWF_BLEND_DEST_OVER_SOURCE,
 TWF_BLEND_SOURCE_IN_DEST,
 TWF_BLEND_DEST_IN_SOURCE,
 TWF_BLEND_SOURCE_HELD_DEST,
 TWF_BLEND_DEST_HELD_SOURCE
};

struct _TWF_File;
typedef struct _TWF_File TWF_File;

struct _TWF_FileOptions;
typedef struct _TWF_FileOptions TWF_FileOptions;

typedef unsigned long TWF_Color;

/*
 * TWF_encodeRGBAColor(r, g, b, alpha)
 *	Creates a new long int value that represents the RGBA color given.
 *	'alpha' must be a value between 0.0 and 1.0, which will be converted
 *	to an 8-bit integer value internally.
 */
TWF_Color TWF_encodeRGBAColor (unsigned char r, unsigned char g,
                                   unsigned char b, double alpha);

/*
 * TWF_decodeRGBAColor(color, r, g, b, alpha)
 *	Fills the given 'r', 'g', 'b' and 'alpha' return variables with the
 *	corresponding components of the color 'color' given. On success,
 *	return TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeRGBAColor (TWF_Color color, unsigned char *r,
                         unsigned char *g, unsigned char *b, double *alpha);

/*
 * TWF_encodeRGBColor(r, g, b)
 *	Creates a new long int value that represents the RGB color given.
 */
TWF_Color TWF_encodeRGBColor (unsigned char r, unsigned char g,
                                  unsigned char b);

/*
 * TWF_decodeRGBColor(color, r, g, b)
 *	Fills the given 'r', 'g' and 'b' return variables with the
 *	corresponding components of the color 'color' given. On success,
 *	return TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeRGBColor (TWF_Color color, unsigned char *r,
                         unsigned char *g, unsigned char *b);

/*
 * TWF_encodeGAColor(gray, alpha)
 *	Creates a new long int value that represents the gray-alpha color
 *	given. Both 'gray' and 'alpha' must be values from 0.0 to 1.0, both
 *	of which will be converted to 16-bit integer values internally.
 */
TWF_Color TWF_encodeGAColor (double gray, double alpha);

/*
 * TWF_decodeGAColor(color, gray, alpha)
 *	Fills the given 'gray' and 'alpha' return variables with the
 *	corresponding components of the color 'color' given. On success,
 *	return TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeGAColor (TWF_Color color, double *gray, double *alpha);

/*
 * TWF_encodeG8Color(gray)
 *	Creates a new long int value that represents the gray color given.
 *	'gray' must range from 0.0 to 1.0, and will be converted to an
 *	8-bit integer value internally.
 */
TWF_Color TWF_encodeG8Color (double gray);

/*
 * TWF_decodeG8Color(color, gray)
 *	Fills the given 'gray' return variable with the
 *	corresponding component of the color 'color' given. On success,
 *	returns TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeG8Color (TWF_Color color, double *gray);

/*
 * TWF_encodeG12Color(gray)
 *	Creates a new long int value that represents the gray color given.
 *	'gray' must range from 0.0 to 1.0, and will be converted to a
 *	12-bit integer value internally.
 */
TWF_Color TWF_encodeG12Color (double gray);

/*
 * TWF_decodeG12Color(color, gray)
 *	Fills the given 'gray' return variable with the
 *	corresponding component of the color 'color' given. On success,
 *	returns TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeG12Color (TWF_Color color, double *gray);

/*
 * TWF_encodeG16Color(gray)
 *	Creates a new long int value that represents the gray color given.
 *	'gray' must range from 0.0 to 1.0, and will be converted to a
 *	16-bit integer value internally.
 */
TWF_Color TWF_encodeG16Color (double gray);

/*
 * TWF_decodeG16Color(color, gray)
 *	Fills the given 'gray' return variable with the
 *	corresponding component of the color 'color' given. On success,
 *	returns TWF_OK. On failure, returns TWF_ERROR.
 */
int TWF_decodeG16Color (TWF_Color color, double *gray);

/*
 * TWF_newFileOptions()
 *	Creates a new set of file creation options, with all values set to
 *	the default, and returns it.
 *
 *	On failure this function returns NULL.
 */
TWF_FileOptions * TWF_newFileOptions( void );

/*
 * TWF_deleteFileOptions()
 *	Frees the memory used by a TWF_FileOptions object.
 */
void TWF_deleteFileOptions( TWF_FileOptions *opt );

/*
 * TWF_setWorldFileOption(opt, xmin, xmax, ymin, ymax)
 *	Sets the world coordinates option of the TWF_FileOptions instance
 *	given. This will only affect the subsequent file creations that use
 *	the given TWF_FileOptions instance.
 *
 *	The default values are xmin=-100.0, xmax=100.0, ymin=-100.0 and
 *	ymax=100.0.
 */
void TWF_setWorldFileOption( TWF_FileOptions *opt, double xmin, double xmax, 
                         double ymin, double ymax );

/*
 * TWF_setDimensionsFileOption(opt, width_mm, height_mm)
 *	Sets the drawing dimensions option of the TWF_FileOptions instance
 *	given. This will only affect the subsequent file creations that use 
 *	the given TWF_FileOptions instance.
 *
 *	The default values are width_mm=100.0 and height_mm=100.0.
 */
void TWF_setDimensionsFileOption( TWF_FileOptions *opt, 
                              double width_mm, double height_mm );

/*
 * TWF_setResolutionFileOption(opt, width_res, height_res)
 *	Sets the quantization resolution option of the TWF_FileOptions instance
 *	given. This will only affect the subsequent file creations that use 
 *	the given TWF_FileOptions instance.
 *
 *	The default values are width_res=4096 and height_res=4096.
 */
void TWF_setResolutionFileOption( TWF_FileOptions *opt, 
                                  long width_res, long height_res );

/*
 * TWF_setQuantizationFileOption(opt, enabled)
 *	Sets the quantization option of the TWF_FileOptions instance
 *	given. This will only affect the subsequent file creations that use 
 *	the given TWF_FileOptions instance.
 *
 *	The default value is enabled=TWF_TRUE, which means that quantization
 *	will be performed on all primitives.
 */
void TWF_setQuantizationFileOption( TWF_FileOptions *opt, int enabled );

/*
 * TWF_setBackgroundFileOption(opt, color)
 *	Sets the background color option of the TWF_FileOptions instance
 *	given. This will only affect the subsequent file creations that use 
 *	the given TWF_FileOptions instance.
 *	This color must be a valid RGB color created by TWF_encodeRGBColor.
 *
 *	The default value is the R=255, G=255, B=255 white color.
 */
void TWF_setBackgroundFileOption( TWF_FileOptions *opt, TWF_Color color );

/*
 * TWF_setCompressionFileOption(opt, comp)
 *	If 'comp' equals TWF_FALSE, no ZLIB compression will be performed on
 *	the generated file. Otherwise, compression will be performed. This will
 *	only affect the subsequent file creations that use the given
 *	TWF_FileOptions instance.
 *
 *	The default value is TWF_TRUE, which indicates that ZLIB compression
 *	will be performed.
 */
void TWF_setCompressionFileOption( TWF_FileOptions *opt, int comp );

/*
 * TWF_create(filename, opt)
 *	Creates a new TWF file with the name 'filename' given. The function
 *	will always create a new file, overwriting if it already exists.
 *	
 *	The first thing the user must do after creating a TWF file is to
 *	create a new layer by calling TWF_beginLayer, or the file will not
 *	be correctly generated.
 *
 *	'opt' represents the options used when creating the file. If NULL is
 *	given, the default options are used. Otherwise, 'opt' must be a valid
 *	TWF_FileOptions instance created by TWF_newFileOptions.
 *
 *	The file name should end in ".twfz" if compression is enabled
 *	(default) or ".twf" otherwise. The library does not enforce this
 *	convention, however. See 'TWF_setCompressionFileOption' for more
 *	information.
 *
 *	If an error occurs, the function returns NULL. Otherwise, it returns
 *	a new valid instance of TWF_File.
 */
TWF_File * TWF_create (const char *filename, const TWF_FileOptions *opt);

/*
 * TWF_close(file)
 *	Finishes the writing of the given TWF file and frees all memory used
 *	by it.
 *
 *	The file is closed and flushed to disk. This function must be called
 *	for every file created by TWF_create or TWF_createRaw or the files will
 *	not be correctly generated.
 *
 *	If all goes well, returns TWF_OK. On error, returns TWF_ERROR.
 */
int TWF_close( TWF_File *file );

/*
 * TWF_getErrorCode(file)
 *	Returns the identification code of the last error that occurred while
 *	writing the given file.
 *
 *	If no error has occurred yet, returns TWF_ERR_NONE.
 */
int TWF_getErrorCode( const TWF_File *fl );

/*
 * TWF_getErrorDescription(file)
 *	Returns a readable description of the last error that occurred while
 *	writing the given file. This string is an internal string that should
 *	not be altered by the caller.
 *
 *	If no error has occurred yet, returns NULL.
 */
const char * TWF_getErrorDescription( const TWF_File *file );

/*
 * TWF_extension(file, parameter, value)
 *	Writes a name-value pair as a TWF extension to the TWF file.
 *
 *	Returns TWF_OK on sucess and TWF_ERROR on failure.
 */
int TWF_extension( TWF_File *file, const char *parameter, const char *value );

/*
 * TWF_beginLayer(file, name, vis, detect)
 *	Begins the definition of a new layer in the TWF file.
 *	'name' is the name of the new layer. This causes all the state
 *	variables to be returned to their default values.
 *
 *	If 'vis' is non-zero, the layer will be visible.
 *
 *	If 'detect' is non-zero, the layer will be detectable.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_beginLayer( TWF_File *file, const char *name, unsigned char vis, 
                    unsigned char detect );

/*
 * TWF_beginObject(file, name)
 *	Begin the definition of a new object with the name 'name' given.
 *	All primitives following this command will be part of this object
 *	until it is closed. An object is closed by calling endObject or
 *	beginLayer.
 *
 *	Object definitions cannot be nested.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_beginObject( TWF_File *file, const char *name );

/*
 * TWF_endObject(file)
 *	Ends the definition of the current object, if any.
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_endObject( TWF_File *file );

/*
 * TWF_begin(file, type, x, y)
 *	Starts the definition of a new polygonal primitive (polyline or
 *	polygon) or a new cubic Bezier chain.
 *
 *	Nothing is actually written to disk until a call to TWF_end is
 *	made.
 *
 *	Definitions cannot be nested.
 *
 *	'x' and 'y' are the starting point of the primitive definition.
 *	'type' indicates which type of primitive is being defined.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_begin( TWF_File *file, int type, double x, double y );

/*
 * TWF_vertex(file, x, y)
 *	Appends the vertex (x,y) to the polygonal primitive being currently
 *	defined. If no polygonal primitive definition is underway, an error
 *	occurs.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the vertex is
 *	discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_vertex( TWF_File *file,  double x, double y );

/*
 * TWF_cubic(file, ctr1_x, ctrl1_y, ctrl2_x, ctrl2_y, end_x, end_y)
 *	Appends a segment to the cubic being currently defined, if any.
 *	If this is the first segment, it starts at the (x,y) point given to
 *	TWF_begin. Otherwise, it begins at the ending point of the previous
 *	segment.
 *	'ctrl1_x' and 'ctrl1_y' are the coordinates of the first control point.
 *	'ctrl2_x' and 'ctrl2_y' are the coordinates of the second control point.
 *	'end_x' and 'end_y' are the coordinates of the ending point.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the cubic
 *	segment is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_cubic( TWF_File *file,  double ctrl1_x, double ctrl1_y,
               double ctrl2_x, double ctrl2_y,
               double end_x, double end_y );

/*
 * TWF_end(file)
 *	Ends the current primitive definition and writes it to the file.
 *	The primitive will comply to the current state attributes.
 *
 *	If no definition is underway, an error occurs.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_end( TWF_File *file );

/*
 * TWF_arc(file, x, y, w, h, start, end)
 *	Fills the interior of an elliptic arc or draws its outline according
 *	to the value last set by TWF_setFill.
 *	'x' and 'y' specify the center of the ellipse.
 *	'w' and 'h' are the width and height of the bounding box of the ellipse.
 *	'start' and 'end' are the starting and ending angles in degrees. Both
 *	angles must range from 0 to 360. If 'start' equals 'end', the full
 *	ellipse will be filled.
 *	The area filled will be area defined by the arc and two straight lines
 *	that connect the edges of the arc to the center of the ellipse.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the arc
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_arc( TWF_File *file, double x, double y, double w, double h,
             double start, double end );

/*
 * TWF_text(file, x, y, text)
 *	Draws the string 'text' at the position (TWF_File *file, 'x', 'y').
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_text( TWF_File *file, double x, double y, const char *text );

/*
 * TWF_beginSetShape(file, w, h, id)
 *	This function begins the definition of a new shape. Shapes are always
 *	defined with their center at (TWF_File *file, 0,0). All primitives
 *	added until the next call to TWF_endSetShape will be added to this
 *	shape.
 *
 *	'w' and 'h' are the width and height, respectively, of the bounding
 *	box of the new shape.
 *
 *	'id' will be set to the identifier of the new shape.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_beginSetShape( TWF_File *file, double w, double h, unsigned long *id );

/*
 * TWF_endSetShape(file)
 *	This function ends the definition of a new shape.
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_endSetShape( TWF_File *file );

/*
 * TWF_shape(file, id, x, y, w, h, rotation)
 *	This function instantiates a shape applying transformations.
 *
 *	'id' is the unique identifier of a previously defined shape.
 *
 *	'x' and 'y' specify the location where the center of the shape will be
 *	rendered.
 *
 *	'rotation' indicates the angle, in degrees, by which the shape will be
 *	rotated around its center in counterclockwise direction.
 *
 *	'w' and 'h' are new width and height of the shape, which may cause it
 *	to be scaled.
 *
 *	The shape will be scaled, rotated and then moved to the desired
 *	location, in that order.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_shape( TWF_File *file, unsigned long id, double x,  double y, 
               double rotation, double w, double h );

/*
 * TWF_stampShape(file, id, x, y)
 *	This function instantiates a shape as defined in a specified location.
 *
 *	'id' is the unique identifier of a previously defined shape.
 *
 *	'x' and 'y' specify location of the center of the shape.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_stampShape( TWF_File *file, unsigned long id, double x, double y );

/*
 * TWF_setImage(file, w, h, colors, id)
 *	This function defines a new raster image and assigns it a new unique 
 *	identifier.
 *
 *	'w' and 'h' are the width and height, in pixels, of the new image.
 *
 *	'colors' is an array of w*h pixel colors, listed line by line, from
 *	top to bottom. Each line is given from left to right.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setImage( TWF_File *file, unsigned short w, unsigned short h, 
                  const TWF_Color *colors, unsigned long *id );

/*
 * TWF_subImage(file, id, sub_x, sub_y, sub_w, sub_h, x, y, w, h)
 *	This function draws a rectangular portion of a previously defined
 *	image in a destination rectangle. The sub-image will be stretched to
 *	fit into the destination rectangle.
 *
 *	'id' is the unique identifier of the image obtained by a previous
 *	call to TWF_setImage.
 *
 *	'sub_x' is the X coordinate of the lower-left corner of the sub-image,
 *	ranging from 0 to the full image's width minus one.
 *
 *	'sub_y' is the Y coordinate of the lower-left corner of the sub-image,
 *	ranging from 0 to the full image's height minus one.
 *
 *	'sub_w' and 'sub_h' are the width and height of the sub-image in pixels.
 *
 *	'x' and 'y' specify the lower-left corner of the destination rectangle.
 *
 *	'w' and 'h' are the width and height of the destination rectangle.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the subimage
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_subImage( TWF_File *file, unsigned long id,
                  unsigned short sub_x, unsigned short sub_y,
                  unsigned short sub_w, unsigned short sub_h,
                  double x, double y, double w, double h );

/*
 * TWF_image(file, id, x, y, w, h)
 *	This function draws a previously defined image in a destination 
 *	rectangle. The sub-image will be stretched to fit into the destination
 *	rectangle if necessary.
 *
 *	'id' is the unique identifier of the image obtained by a previous
 *	call to TWF_setImage.
 *
 *	'x' and 'y' specify the lower-left corner of the destination rectangle.
 *
 *	'w' and 'h' are the width and height of the destination rectangle.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the image 
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_image( TWF_File *file, unsigned long id, double x, double y, 
               double w, double h );

/*
 * TWF_mark(file, x, y)
 *	Draws a symbol at a specified position.
 *
 *	'x' and 'y' specificy the position of the center of the symbol.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_mark( TWF_File *file, double x, double y );

/*
 * TWF_line(file, x1, y1, x2, y2)
 *	Draws a line between points (x1, y1) and (x2, y2).
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the line
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_line( TWF_File *file, double x1, double y1, double x2, double y2 );

/*
 * TWF_box(file, x, y, w, h)
 *	Fills the interior or draws the ouline of a box depending on the value
 *	last set by TWF_setFill.
 *
 *	'x' and 'y' specify the position of the lower-left corner of the box.
 *
 *	'w' and 'h' are the width and height of the box.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the box
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_box( TWF_File *file, double x, double y, double w, double h );

/*
 * TWF_circle(file, x, y, radius)
 *	Draws the ouline of a circle or fills its interior depending on the
 *	value last set by TWF_setFill.
 *
 *	'x' and 'y' specify the center of the circle.
 *
 *	'radius' specifies the radius of the circle.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_circle( TWF_File *file, double x, double y, double radius );

/*
 * TWF_ellipse(file, x, y, w, h)
 *	Draws the ouline or fills the interior of an ellipse depending on the
 *	last value set by TWF_setFill.
 *
 *	'x' and 'y' specify the center of the ellipse.
 *
 *	'w' and 'h' specify the width and height of the smallest bounding box
 *	which contains the ellipse.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_ellipse( TWF_File *file, double x, double y, double w, double h );

/*
 * TWF_beginSetEdge(file, edge_id)
 *	Begins the definition of a new edge.
 *
 *	'edge_id' will be set to the unique identifier of the new edge if
 *	successful.
 *
 *	'x' and 'y' specify the starting point of the edge.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_beginSetEdge( TWF_File *file, long *edge_id, double x, double y );

/*
 * TWF_vertexSetEdge(file, x, y)
 *	Adds another vertex to the edge being currently defined, if any.
 *	'x' and 'y' specify the new vertex.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the vertex 
 *	is discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_vertexSetEdge( TWF_File *file, double x, double y );

/*
 * TWF_endSetEdge(file)
 *	Ends the new edge definition and commits it to the file.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure. If the edge is
 *	discarded by the quantization process, returns TWF_DISCARD.
 */
int TWF_endSetEdge( TWF_File *file );

/*
 * TWF_edge(file, egde_id)
 *	Draws a previously defined edge whose unique identifier is given in 
 *	'edge_id'.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_edge( TWF_File *file, long edge_id );

/*
 * TWF_edgeRange(file, start, end)
 *	Draws a series of previously defined edges that have identifiers
 *	ranging from 'start' to 'end'.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_edgeRange( TWF_File *file, short start, short end );

/*
 * TWF_beginSetFace(file, face_id)
 *	Begins the definition of a new face.
 *
 *	'face_id' will be set to the unique identifier of the new face if 
 *	successful.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_beginSetFace( TWF_File *file, unsigned long *face_id );

/*
 * TWF_edgeSetFace(file, edge_id, reversed)
 *	Adds a new edge to the contour of the face.
 *
 *	'edge_id' is the unique identifier of the edge to be added.
 *
 *	'reversed' indicates wether the edge vertexes should be traversed in 
 *	definition order (TWF_FALSE) or in reverse definition order (TWF_TRUE).
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_edgeSetFace( TWF_File *file, long edge_id, int reversed );

/*
 * TWF_holeSetFace(file, hole_id)
 *	Adds a new hole to a face.
 *
 *	'hole_id' is the unique identifier of a previously defined face
 *	whose interior should be subtracted from the area of the face being 
 *	defined.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_holeSetFace( TWF_File *file, unsigned long hole_id );

/*
 * TWF_endSetFace(file)
 *	Ends the current face definition and commits it to file.
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_endSetFace( TWF_File *file );

/*
 * TWF_face(file, face_id)
 *	Fills the interior of a face.
 *
 *	'face_id' is the unique identifier of a face obtained by a call to
 *	TWF_beginSetFace.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_face( TWF_File *file, unsigned long face_id );

/*
 * TWF_setColorSize(file, mode)
 *	Sets the new color mode used for all color types in the current
 *	context.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setColorSize( TWF_File *file, unsigned char mode );

/*
 * TWF_getColorSize(file)
 *	Returns the current color mode for all color types in the current
 *	context.
 */
unsigned char TWF_getColorSize( const TWF_File *file );

/*
 * TWF_setBlendMode(file, mode)
 *	Sets the new blend mode used for all color types in the current
 *	context.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setBlendMode( TWF_File *file, unsigned char mode );

/*
 * TWF_getBlendMode(file)
 *	Returns the current blend mode for all color types in the current
 *	context.
 */
unsigned char TWF_getBlendMode( const TWF_File *file );

/*
 * TWF_setLineColor(file, color)
 *	Sets the color used to draw primitive outlines in the current
 *	context.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setLineColor( TWF_File *file, TWF_Color color );

/*
 * TWF_getLineColor(file)
 *	Fetches the color used to draw primitive outlines in the current
 *	context.
 *
 *	'color' will be set to the current line color.
 *
 *	'color_size' will be set to the color mode this color belongs to.
 */
void TWF_getLineColor( const TWF_File *file, TWF_Color *color, 
                       unsigned char *color_size );

/*
 * TWF_setLineDashes(file, dash)
 *	Sets the dashing style used to draw primitive outlines in the
 *	current context.
 *
 *	'dash' is a pattern meaning which segments must be drawn. Each bit
 *	represents if a 16th of a unit must be drawn or not.
 *	Examples:
 *		0x00FF  "        --------"
 *		0xAAAA  "- - - - - - - - "
 *		0x8000  "-               "
 *		0xD54D  "---  - - -  --- "
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setLineDashes( TWF_File *file, unsigned short dash );

/*
 * TWF_getLineDashes(file)
 *	Returns the dashing style used to draw primitive outlines in the
 *	current context.
 */
unsigned short TWF_getLineDashes( const TWF_File *file );

/*
 * TWF_setLineStyle(file, joint, begin, end, weight)
 *	Sets the style and decorations used to draw primitive outlines in the
 *	current context.
 *
 *	'joint' is the line joint style.
 *
 *	'begin' is the decoration used on the beginning of a line.
 *
 *	'end' is the decoration used on the end of a line.
 *
 *	'weight' is the line thickness.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setLineStyle( TWF_File *file, int joint, int begin,
                      int end, unsigned char weight );

/*
 * TWF_getLineStyle(file, joint, begin, end, weight)
 *	Gets the style and decorations used to draw primitive outlines in the
 *	current context.
 *
 *	'joint' is the line joint style. It must match one of the
 *	constants of enum TWF_JOINT_STYLE.
 *
 *	'begin' is the decoration used on the beginning of a line.
 *	It must match one of the constants of enum TWF_LINE_DECORATION_STYLE.
 *
 *	'end' is the decoration used on the end of a line.
 *	It must match one of the constants of enum TWF_LINE_DECORATION_STYLE.
 *
 *	'weight' is the line thickness.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
void TWF_getLineStyle( const TWF_File *file, int *joint, int *begin, int *end,
                       unsigned char *weight );

/*
 * TWF_setMarkColor( file, color )
 *	Sets the color used to draw marks in the current context.
 *
 *	'color' is the new mark color.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setMarkColor( TWF_File *file, TWF_Color color );

/*
 * TWF_getMarkColor( file, color, color_size )
 *	Gets the color used to draw marks from the 
 *	current context.
 *
 *	'color' will be set to the mark color.
 *
 *	'color_size' will be set to the type of color filled in 'color'
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
void TWF_getMarkColor( const TWF_File *file, TWF_Color *color, 
                       unsigned char *color_size );

/*
 * TWF_setMarkStyle( file, type, size )
 *	Sets the type of mark drawn with TWF_mark function and it's size
 *
 *	'type' is the new the mark type.
 *
 *	'size' is the new mark size, in world-coordinate units.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setMarkStyle( TWF_File *file, int type, unsigned long size );

/*
 * TWF_getMarkStyle( file, type, size )
 *	Gets the mark style information used to draw marks in the current 
 *	context.
 *
 *	'type' will be set to the mark type
 *
 *	'size' will be set to the mar size
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
void TWF_getMarkStyle( const TWF_File *file, int *type, unsigned long *size );

/*
 * TWF_setFill(file, enabled)
 *	Enables or disables the filling of the interior of primitives.
 *
 *	If 'enabled' equals TWF_FALSE, then primitives will have their outlines
 *	drawn using the current line style. If 'enabled' equals TWF_TRUE,
 *	only the interior of the primitives will be filled with the
 *	current fill style.
 *
 *	The default fill value is TWF_FALSE.
 *
 *	This function returns the previous fill value.
 */
int TWF_setFill( TWF_File *file, int enabled );

/*
 * TWF_getFill(file);
 *	Returns the current state of the filling of primitives, which is
 *	either TWF_TRUE or TWF_FALSE.
 */
int TWF_getFill( const TWF_File *file );

/*
 * TWF_setFillColor(file, color)
 *	Changes the fill style for the current context to the color 'color'
 *	given.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setFillColor( TWF_File *file, TWF_Color color );

/*
 * TWF_getFillColor(file, color, color_size)
 *	Fetches the fill style for the current context and stores it in 'color'.
 *	'color_size' will be set to the color size of the color.
 */
void TWF_getFillColor( const TWF_File *file, TWF_Color *color,
                       unsigned char *color_size );

/*
 * TWF_setGradientFill(file, x1, y1, c1, x2, y2, c2)
 *	Changes the fill style for the current context to a gradient variation
 *	of colors.
 *
 *	'x1' and 'y1' specify the first reference point, where the color should
 *	be 'c1'.
 *
 *	'x2' and 'y2' specify the second reference point, where the color should
 *	be 'c2'.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setGradientFill( TWF_File *file, double x1, double y1, TWF_Color c1,
                         double x2, double y2, TWF_Color c2 );

/*
 * TWF_getGradientFill(file, x1, y1, c1, x2, y2, c2)
 *	Fetches the gradient fill style for the current context.
 *
 *	'x1' and 'y1' specify the first reference point, where the color should
 *	be 'c1'.
 *
 *	'x2' and 'y2' specify the second reference point, where the color should
 *	be 'c2'.
 *
 *	'color_size' is the color size 'c1' and 'c2' belong to.
 */
void TWF_getGradientFill( const TWF_File *file,
                          double *x1, double *y1, TWF_Color *c1, 
                          double *x2, double *y2, TWF_Color *c2,
                          unsigned char *color_size );

/*
 * TWF_setTextureFill(file, x, y, image_id)
 *	Changes the fill style for the current context to the tiled repetition
 *	of a previously defined image.
 *
 *	'x' and 'y' specify the lower-left corner of one of the image
 *	repetitions.
 *
 *	'image_id' is the unique identifier of the image obtained by a call to
 *	TWF_setImage.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setTextureFill( TWF_File *file, double x, double y, 
                        unsigned long image_id );

/*
 * TWF_getTextureFill(file, x, y, image_id)
 *	Fetches the current texture fill parameters.
 *
 *	'x' and 'y' will be set to the lower-left corner of one of the image
 *	repetitions.
 *
 *	'image_id' will be set to the identifier of the texture image.
 *	TWF_setImage.
 */
void TWF_getTextureFill( const TWF_File *file, double *x, double *y,
                         unsigned long *image_id );

/*
 * TWF_setStretchedTextureFill(file, x, y, w, h, image_id)
 *	Changes the fill style for the current context to the tiled repetition
 *	of a previously defined image.
 *
 *	'x' and 'y' specify the lower-left corner of one of the image
 *	repetitions.
 *
 *	'w' and 'h' specify the width and height of each tile.
 *
 *	'image_id' is the unique identifier of the image obtained by a call to
 *	TWF_setImage.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setStretchedTextureFill( TWF_File *file, double x, double y, 
                                 double w, double h, unsigned long image_id );

/*
 * TWF_getStretchedTextureFill(file, x, y, w, h, image_id)
 *	Fetches the current stretched texture fill parameters.
 *
 *	'x' and 'y' specify the lower-left corner of one of the image
 *	repetitions.
 *
 *	'w' and 'h' specify the width and height of each tile.
 *
 *	'image_id' is the unique identifier of the image obtained by a call to
 *	TWF_setImage.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
void TWF_getStretchedTextureFill( const TWF_File *file, double *x, double *y,
                                  double *w, double *h, 
                                  unsigned long *image_id );

/*
 * TWF_setImageStretchMode(file, mode)
 *	Changes the image stretch mode for the current context to the value
 *	'mode' given.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setImageStretchMode( TWF_File *file, unsigned char mode );

/*
 * TWF_setImageStretchMode(file, mode)
 *	Returns the image stretch mode for the current context.
 */
unsigned char TWF_getImageStretchMode( const TWF_File *file );

/*
 * TWF_setTextColor(file, color)
 *	Sets the new color used to draw text in the current context.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setTextColor( TWF_File *file, TWF_Color color );

/*
 * TWF_getTextColor(file, color, color_size)
 *	Fetches the color used to draw text in the current context and stores
 *	it in 'color'.
 *
 *	'color_size' is the size of the color 'color'.
 */
void TWF_getTextColor( const TWF_File *file, TWF_Color *color, 
                       unsigned char *color_size );

/*
 * TWF_setFont(file, typeface, style, size)
 *	Sets the font used to draw text in the current context.
 *
 *	'typeface' is the font family desired.
 *
 *	'style' is the font style desired.
 *
 *	'size' is the font size desired.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setFont( TWF_File *file, unsigned char typeface, unsigned char style,
                 unsigned long size );

/*
 * TWF_getFont(file, typeface, style, size)
 *	Fetches the font used to draw text in the current context.
 *
 *	'typeface' will be set to the current font family.
 *
 *	'style' will be set to the current font style.
 *
 *	'size' will be set to the current font size.
 */
void TWF_getFont( const TWF_File *file, unsigned char *typeface,
                  unsigned char *style, unsigned char *size );

/*
 * TWF_setTextAlignment(file, alignment, direction)
 *	Defines the alignment used to draw text in the current context.
 *
 *	'alignment' is a constant describing the position of the reference
 *	point in relation to the text drawn in TWF_text.
 *
 *	'direction' is angle by which to rotate the text around the reference
 *	point, in degrees.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setTextAlignment( TWF_File *file, unsigned char alignment, 
                          double direction );

/*
 * TWF_getTextAlignment(file, alignment, direction)
 *	Fetches the alignment used to draw text from the current context.
 *
 *	'alignment' will be set to a constant describing the position of the
 *	reference point in relation to the text drawn in TWF_text.
 *
 *	'direction' will be set to the angle by which text is rotated around
 *	the reference point, in degrees.
 */
void TWF_getTextAlignment( const TWF_File *file, unsigned char *alignment,
                           double *direction );

/*
 * TWF_setPalette(file, ncolors, colors)
 *	Sets the current palette used for indexed color modes.
 *
 *	'ncolors' is the number of colors in the new palette, ranging from 1
 *	to 256.
 *
 *	'colors' is the array of colors, encoded in the current color mode,
 *	with 'ncolors' elements.
 *
 *	Returns TWF_OK on success and TWF_ERROR on failure.
 */
int TWF_setPalette( TWF_File *file, int ncolors, const TWF_Color *colors );

/*
 * TWF_getPalette(file, ncolors, colors)
 *	Fetches the current palette used for indexed color modes.
 *
 *	'ncolors' will be set to the length of the current palette, ranging
 *	from 1 to 256.
 *
 *	'colors' must be a pointer to a previously allocated array of at
 *	least 256 instances of TWF_Color (256 * sizeof(TWF_Color) bytes).
 *	It will be set with the colors in the palette in their respective
 *	order.
 *
 *	'color_size' will be set to the color size of all the components
 *	of the palette.
 */
void TWF_getPalette( const TWF_File *file, int *ncolors, TWF_Color *colors,
                     unsigned char *color_size );

#ifdef __cplusplus
}
#endif

#endif
